@@ -3,7 +3,7 @@ package POE::Wheel::SocketFactory;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.293'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.299'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw( carp croak );
 use Symbol qw( gensym );
@@ -13,12 +13,14 @@ use Errno qw(
   EWOULDBLOCK EADDRNOTAVAIL EINPROGRESS EADDRINUSE ECONNABORTED
   ESPIPE
 );
+
 use Socket qw(
   AF_INET SOCK_STREAM SOL_SOCKET AF_UNIX PF_UNIX 
   PF_INET SOCK_DGRAM SO_ERROR unpack_sockaddr_in 
   unpack_sockaddr_un PF_UNSPEC SO_REUSEADDR INADDR_ANY 
   pack_sockaddr_in pack_sockaddr_un inet_aton SOMAXCONN
 );
+
 use IO::Handle ();
 use FileHandle ();
 use POE qw( Wheel );
@@ -41,25 +43,37 @@ sub MY_SOCKET_TYPE     () { 10 }
 sub MY_STATE_ERROR     () { 11 }
 sub MY_SOCKET_SELECTED () { 12 }
 
-# Fletch has subclassed SSLSocketFactory from SocketFactory.  He's
-# added new members after MY_SOCKET_SELECTED.  Be sure, if you extend
-# this, to extend add stuff BEFORE MY_SOCKET_SELECTED or let Fletch
-# know you've broken his module.
+# Fletch has subclassed SSLSocketFactory from SocketFactory.
+# He's added new members after MY_SOCKET_SELECTED.  Be sure, if you
+# extend this, to extend add stuff BEFORE MY_SOCKET_SELECTED or let
+# Fletch know you've broken his module.
 
-# Provide dummy constants for systems that don't have them.
-# Test and provide for each constant separately, per suggestion in
-# rt.cpan.org 27250.
+# If IPv6 support can't be loaded, then provide dummies so the code at
+# least compiles.  Suggested in rt.cpan.org 27250.
 BEGIN {
-  eval { require Socket6 };
+  eval "use Socket::GetAddrInfo qw(:newapi getaddrinfo getnameinfo)";
   if ($@) {
-    *Socket6::AF_INET6 = sub () { ~0 };
-    *Socket6::PF_INET6 = sub () { ~0 };
+    my $why = $@;
+    $why =~ s/ at \(eval.* line \d+\.\s*//;
+    *getaddrinfo = sub { return "Socket::GetAddrInfo not loaded: $why" };
+    *getnameinfo = sub { return "Socket::GetAddrInfo not loaded: $why" };
   }
-  else {
-    eval { my $x = &Socket6::AF_INET6 };
-    *Socket6::AF_INET6 = sub () { ~0 } if $@;
-    eval { my $x = &Socket6::PF_INET6 };
-    *Socket6::PF_INET6 = sub () { ~0 } if $@;
+
+  # Socket6 provides AF_INET6 and PF_INET6 where earlier Perls' Socket don't.
+  eval "use Socket qw(AF_INET6)";
+  if ($@) {
+    eval "use Socket6 qw(AF_INET6)";
+    if ($@) {
+      *AF_INET6 = sub { ~0 };
+    }
+  }
+
+  eval "use Socket qw(PF_INET6)";
+  if ($@) {
+    eval "use Socket6 qw(PF_INET6)";
+    if ($@) {
+      *PF_INET6 = sub { ~0 };
+    }
   }
 }
 
@@ -76,8 +90,8 @@ sub DOM_INET6 () { 'inet6' }  # INET v6 domain socket
 my %map_family_to_domain = (
   AF_UNIX,  DOM_UNIX,  PF_UNIX,  DOM_UNIX,
   AF_INET,  DOM_INET,  PF_INET,  DOM_INET,
-  &Socket6::AF_INET6, DOM_INET6,
-  &Socket6::PF_INET6, DOM_INET6,
+  AF_INET6, DOM_INET6,
+  PF_INET6, DOM_INET6,
 );
 
 sub SVROP_LISTENS () { 'listens' }  # connect/listen sockets
@@ -174,7 +188,8 @@ sub _define_accept_state {
         }
         elsif ( $domain eq DOM_INET6 ) {
           $peer = getpeername($new_socket);
-          ($peer_port, $peer_addr) = Socket6::unpack_sockaddr_in6($peer);
+          ((my $error), $peer_port, $peer_addr) =  getnameinfo($peer);
+          warn $error if $error;
         }
         else {
           die "sanity failure: socket domain == $domain";
@@ -293,10 +308,9 @@ sub _define_connect_state {
       # INET6 socket stacks tend not to.
       elsif ($domain eq DOM_INET6) {
         if (defined $peer) {
-          eval {
-            ($peer_port, $peer_addr) = Socket6::unpack_sockaddr_in6($peer);
-          };
-          if (length $@) {
+          ((my $error), $peer_port, $peer_addr) = getnameinfo($peer);
+          if ($error) {
+            warn $error;
             $peer_port = $peer_addr = undef;
           }
         }
@@ -780,7 +794,7 @@ sub new {
       $bind_address = (
         (defined $params{BindAddress})
         ? $params{BindAddress}
-        : Socket6::in6addr_any()
+        : "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"  # XXX - Only Socket6 has?
       );
 
       # Set the bind port, or default to 0 (any) if none specified.
@@ -802,12 +816,16 @@ sub new {
       BEGIN { eval { require bytes } and bytes->import; }
 
       # Resolve the bind address.
-      my @info = Socket6::getaddrinfo(
-        $bind_address, $bind_port,
-        $self->[MY_SOCKET_DOMAIN], $self->[MY_SOCKET_TYPE],
+      my ($error, @addresses) = getaddrinfo(
+        $bind_address, $bind_port, {
+          family   => $self->[MY_SOCKET_DOMAIN],
+          socktype => $self->[MY_SOCKET_TYPE],
+        }
       );
 
-      if (@info < 5) {  # unless defined $bind_address
+      unless (@addresses) {
+        warn $error if $error;
+
         $! = EADDRNOTAVAIL;
         $poe_kernel->yield(
           $event_failure,
@@ -816,7 +834,7 @@ sub new {
         return $self;
       }
 
-      $bind_address = $info[3];
+      $bind_address = $addresses[0]->{addr};
     }
   }
 
@@ -904,16 +922,19 @@ sub new {
         $error_tag = "inet_aton";
       }
       elsif ($abstract_domain eq DOM_INET6) {
-        my @info = Socket6::getaddrinfo(
-          $params{RemoteAddress}, $remote_port,
-          $self->[MY_SOCKET_DOMAIN], $self->[MY_SOCKET_TYPE],
+        my ($error, @addresses) = getaddrinfo(
+          $params{RemoteAddress}, $remote_port, {
+            family   => $self->[MY_SOCKET_DOMAIN],
+            socktype => $self->[MY_SOCKET_TYPE],
+          },
         );
 
-        if (@info < 5) {
+        unless (@addresses) {
+          warn $error if $error;
           $connect_address = undef;
         }
         else {
-          $connect_address = $info[3];
+          $connect_address = $addresses[0]->{addr};
         }
 
         $error_tag = "getaddrinfo";
@@ -1257,9 +1278,12 @@ POE::Wheel::SocketFactory contains a table of supported domains and
 the instructions needed to create them.  Please send patches to
 support additional domains, as needed.
 
-Note: C<AF_INET6> and C<PF_INET6> are supplied by the L<Socket6>
-module, which is available on the CPAN.  You must have Socket6 loaded
-before SocketFactory can create IPv6 sockets.
+Note: C<AF_INET6> and C<PF_INET6> are supplied by the L<Socket>
+module included in Perl 5.8.0 or later.  Perl versions before 5.8.0
+should not attempt to use IPv6 until someone contributes a workaround.
+
+IPv6 support requires a 21st century Socket module and the presence of
+Socket::GetAddrInfo to resolve host names to IPv6 addresses.
 
 TODO - Example.
 
@@ -1522,7 +1546,9 @@ listening or connecting.  See below for the differences.
 
 For INET sockets, C<$_[ARG1]> and C<$_[ARG2]> hold the socket's remote
 address and port, respectively.  The address is packed; see
-L<Socket/inet_ntoa> if a human-readable version is needed.
+L<Socket/inet_ntoa> if a human-readable IPv4 address is needed.
+L<Socket::GetAddrInfo/getnameinfo> provides numeric addresses for IPv4
+and IPv6 addresses.
 
   sub handle_new_client {
     my $accepted_socket = $_[ARG0];
@@ -1581,9 +1607,12 @@ A sample FailureEvent handler:
 L<POE::Wheel> describes the basic operations of all wheels in more
 depth.  You need to know this.
 
-L<Socket6> is required for IPv6 work.  POE::Wheel::SocketFactory will
-load it automatically if it's installed, but applications will need to
-use it themselves to get access to AF_INET6.
+L<Socket::GetAddrInfo> is required for IPv6 work.
+POE::Wheel::SocketFactory will load it automatically if it's
+installed.  SocketDomain => AF_INET6 is required to trigger IPv6
+behaviors.  AF_INET6 is exported by the Socket module on all but the
+oldest versions of Perl 5.  If your Socket doesn't provide AF_INET6,
+try installing Socket6 instead.
 
 The SEE ALSO section in L<POE> contains a table of contents covering
 the entire POE distribution.
